//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  special.cpp
//! \brief NSpecial: this and that, written ad hoc.

#include "defs.hpp"

#include <cmath>

#include <fftw3.h>

#include "../readplus/ask.hpp"

#include "olf.hpp"
#include "mem.hpp"
#include "fsel.hpp"
#include "loop.hpp"
#include "obj.hpp"
#include "slice.hpp"
#include "special.hpp"

namespace NSpecial {
    void q_eval( double *par, int m_dat, double *fvec, void *data, int *info);
    void lm_print(int n_par, double *par, int m_dat, double *fvec,
                  void *data, int iflag, int iter, int nfev);
    double q_kernel( double t, double *p );
}

void NSpecial::FourierCosine()
{
    fftw_plan plan;
    int n;

    FileIterator fiter(SFSel::instance()->selD());
    while( const COld *fin = fiter.nextD() ) {
        POld fout( fin->new_POld() );
        fout->log_action("Fourier Cosine");

        cout << "WARNING: coordinates not changed\n";
        cout << "WARNING: x not changed\n";
        cout << "WARNING: dy not calculated\n";
        // *** set coordinates ***
//                olf.xco = CCoord("z", "m");
//                olf.yco = CCoord("B_dim", "B-unit");

        for ( int j=0; j<fin->nJ(); j++ ) {
            CSpec* sin = fin->VS(j);
            PSpec sout( new CSpec( *sin ) );

            n = sin->size();

            plan = fftw_plan_r2r_1d(
                n, &(sin->y[0]), &(sout->y[0]),
                FFTW_REDFT00,  FFTW_ESTIMATE);
            fftw_execute( plan );

            sout->dy.clear();
            for ( int i=0; i<n; ++i ) {
                sout->y[i] /= sqrt(2.*n);
            }

            fout->V.push_back( move(sout) );
        }
        SMem::instance()->mem_store( move(fout) );
    }
}

void NSpecial::Test()
{
    // relative error in form factor

    POld fout( new COld );

    fout->name = "TEST_";
    fout->xco = CCoord("a", "");
    fout->yco = CCoord("FF", "");
    fout->ZCo.push_back( CCoord("b", "") );

    static int Nj = 100;
    static int Ni = 1000;
    
    for ( int j=0; j<Nj; ++j ){
        double b = pow( 1.5, j-Nj );
        PSpec s( new CSpec );
        s->z.push_back( PObjDbl( new CObjDbl (b) ) );
        s->resize(Ni,false);
        for ( int i=0; i<Ni; ++i ){
            double a = b + pow( 1.2, i-Ni );
            s->x[i] = a;
            complex<long double> ff1 = ff_test( (long double) a, (long double) b );
            complex<double> ff2 = ff_test( a, b );
            complex<double> diff = (complex<double>)(ff1)-ff2;
            s->y[i] = std::abs(diff)/std::abs(ff1);
        }
        fout->V.push_back( move(s) );
    }
    SMem::instance()->mem_store( move(fout) );
}
